@@ -50,6 +50,15 @@ extern gboolean todo_style_changed;
 extern gint todo_current_sort_column;
 extern gint todo_current_sort_type;
 
+/* alarm stuff */
+extern CalendarAlarm alarm_defaults[4];
+extern gboolean beep_on_display;
+extern gboolean enable_aalarm_timeout;
+extern guint audio_alarm_timeout;
+extern const guint MAX_AALARM_TIMEOUT;
+extern gboolean enable_snooze;
+extern guint snooze_secs;
+extern const guint MAX_SNOOZE_SECS;
 
 /* Creates and runs the preferences dialog box */
 void properties (GtkWidget *toplevel);
@@ -81,4 +90,12 @@ char *color_spec_from_prop (ColorProp propnum);
 
 void save_default_calendar (GnomeCalendar *gcal);
 
+GnomeCalendar *new_calendar (char *full_name, char *calendar_file,
+			     char *geometry, char *page, gboolean hidden);
+
 #endif
+
+
+
+
+
