@@ -23,13 +23,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <glib.h>
 
 #include "card.h"
 #include "my.h"
 #include "pairs.h"
 
 #define is_a_prop_of(obj,prop) isAPropertyOf (obj,prop)
-#define str_val(obj) the_str = fakeCString (vObjectUStringZValue (obj))
+#define str_val(obj) the_str = (vObjectValueType(obj))? fakeCString (vObjectUStringZValue (obj)) : calloc(1, 1)
 #define has(obj,prop) (vo = isAPropertyOf (obj, prop))
 
 static VObject *card_convert_to_vobject(Card *crd);
@@ -55,6 +56,7 @@ card_prop_empty(void)
 	prop.charset = NULL;
 	prop.lang = NULL;
 	prop.grp = NULL;
+	prop.xtension = NULL;
 	
 	prop.user_data = NULL;
 	
@@ -88,6 +90,7 @@ extern void
 card_prop_free(CardProperty prop)
 {
 	CardGroup *node, *node2;
+	GList *l;
 	
 	MY_FREE(prop.charset);
 	MY_FREE(prop.lang);
@@ -97,6 +100,14 @@ card_prop_free(CardProperty prop)
 		free(node);
 	}
 	
+	for (l = prop.xtension; l; l = l->next) {
+		CardXAttribute *xa = (CardXAttribute *) l->data;
+		g_free (xa->name);
+		MY_FREE (xa->data);
+	}
+	
+	g_list_free (l);
+	
 	prop.used = FALSE;
 }
 	
@@ -117,6 +128,7 @@ card_new(void)
 	c->dellabel.l = NULL;
 	c->phone.l    = NULL;
 	c->email.l    = NULL;
+	c->xtension.l = NULL;
 	c->timezn.sign  = 1;
 	c->timezn.hours = c->timezn.mins = 0;
 	c->geopos.lon   = c->geopos.lat  = 0;
@@ -142,7 +154,7 @@ card_new(void)
 	c->name.prop   = c->photo.prop = c->bday.prop    = c->timezn.prop   = 
 	c->geopos.prop = c->logo.prop  = c->org.prop     = c->rev.prop      =
 	c->sound.prop  = c->key.prop   = c->deladdr.prop = c->dellabel.prop =
-	c->phone.prop  = c->email.prop = c->prop = card_prop_empty();
+	c->phone.prop  = c->email.prop = c->xtension.prop = c->prop = card_prop_empty();
 	
 	c->prop.type = PROP_CARD;
 	c->fname.prop.type = PROP_FNAME;
@@ -154,6 +166,7 @@ card_new(void)
 	c->dellabel.prop.type = PROP_DELLABEL_LIST;
 	c->phone.prop.type = PROP_PHONE_LIST;
 	c->email.prop.type = PROP_EMAIL_LIST;
+	c->xtension.prop.type = PROP_XTENSION_LIST;
 	c->mailer.prop.type = PROP_MAILER;
 	c->timezn.prop.type = PROP_TIMEZN;
 	c->geopos.prop.type = PROP_GEOPOS;
@@ -387,42 +400,84 @@ get_email_type(VObject *o)
 static CardProperty 
 get_CardProperty(VObject *o)
 {
-	VObject *vo;
-	char *the_str;
+	VObjectIterator i;
 	CardProperty prop;
 
 	prop = card_prop_empty();
 	prop.used = TRUE;
 	
-	if (has (o, VCValueProp)) {
-		if (has (vo, VCContentIDProp))
-		  prop.value = VAL_CID;
-		else if (has (vo, VCURLValueProp))
-		  prop.value = VAL_URL;
-	}
+	initPropIterator (&i, o);
+	while (moreIteration (&i)) {
+		VObject *vo = nextVObject (&i);
+		const char *n = vObjectName (vo);
+		int propid;
 	
-	if (has (o, VCEncodingProp)) {
-		if (has (vo, VCQuotedPrintableProp))
-		  prop.encod = ENC_QUOTED_PRINTABLE;
-		else if (has (vo, VC8bitProp))
-		  prop.encod = ENC_8BIT;
-		else if (has (vo, VCBase64Prop))
-		  prop.encod = ENC_BASE64;
-	} else if (has (o, VCQuotedPrintableProp))
-		  prop.encod = ENC_QUOTED_PRINTABLE;
-	else if (has (o, VC8bitProp))
-	  prop.encod = ENC_8BIT;
-	else if (has (o, VCBase64Prop))
-	  prop.encod = ENC_BASE64;
+		propid = card_lookup_name(n);
+		
+		switch (propid) {
+			
+		 case PROP_VALUE:
+		
+			if (has (vo, VCContentIDProp))
+			  prop.value = VAL_CID;
+			else if (has (vo, VCURLValueProp))
+			  prop.value = VAL_URL;
+			break;
+			
+		 case PROP_ENCODING:
+			if (has (vo, VCQuotedPrintableProp))
+			  prop.encod = ENC_QUOTED_PRINTABLE;
+			else if (has (vo, VC8bitProp))
+			  prop.encod = ENC_8BIT;
+			else if (has (vo, VCBase64Prop))
+			  prop.encod = ENC_BASE64;
+			break;
+			
+		 case PROP_QUOTED_PRINTABLE:
+			prop.encod = ENC_QUOTED_PRINTABLE;
+			break;
+			
+		 case PROP_8BIT:
+			prop.encod = ENC_8BIT;
+			break;
+			
+		 case PROP_BASE64:
+			prop.encod = ENC_BASE64;
+			break;
 	
-	if (has (o, VCLanguageProp)) {
-		prop.lang = g_strdup(str_val(vo));
-		free(the_str);
-	}
-	
-	if (has (o, VCCharSetProp)) {
-		prop.charset = g_strdup(str_val(vo));
-		free(the_str);
+		 case PROP_LANG:
+			if (vObjectValueType(vo)) {
+				prop.lang = 
+				  g_strdup(vObjectStringZValue (vo));
+			} else
+			  g_warning ("? < No value for LANG attribute.");
+			break;
+			
+		 case PROP_CHARSET:
+			if (vObjectValueType(vo)) {
+				prop.charset = 
+				  g_strdup(vObjectStringZValue (vo));
+				g_warning(prop.charset); 
+			} else
+			  g_warning ("? < No value for CHARSET attribute.");
+			break;
+		 default:
+				{
+					CardXAttribute *c;
+
+					c = malloc (sizeof (CardXAttribute));
+					c->name = g_strdup (n);
+					
+					if (vObjectValueType(vo))
+					  c->data = 
+					  g_strdup(vObjectStringZValue (vo));
+					else
+					  c->data = NULL;
+					
+					prop.xtension = 
+					  g_list_append(prop.xtension, c);
+				}
+		}
 	}
 
 	return prop;
@@ -523,7 +578,6 @@ get_CardDelAddr(VObject *o)
 	  addr->data[i] = NULL;
 	
 	addr->type    = get_addr_type(o);
-	addr->prop    = get_CardProperty(o);
 
 	if (has (o, VCPostalBoxProp)) {
 		addr->data[PO] = g_strdup(str_val(vo));
@@ -567,7 +621,6 @@ get_CardDelLabel(VObject *o)
 	
 	dellabel->type = get_addr_type(o);
 	dellabel->data = g_strdup(str_val(o));
-	dellabel->prop = get_CardProperty(o);
 	
 	free(the_str);
 	return dellabel;
@@ -582,7 +635,6 @@ get_CardPhone(VObject *o)
 	ret = malloc(sizeof(CardPhone));
 	ret->type = get_phone_type(o);
 	ret->data = g_strdup(str_val(o));
-	ret->prop = get_CardProperty(o);
 	
 	free(the_str);
 
@@ -598,7 +650,6 @@ get_CardEMail(VObject *o)
 	ret = malloc(sizeof(CardEMail)); 
 	ret->type = get_email_type(o);
 	ret->data = g_strdup(str_val(o));
-	ret->prop = get_CardProperty(o);
 	
 	free(the_str);
 	
@@ -711,6 +762,20 @@ get_CardOrg(VObject *o)
 	return org;
 }
 
+static CardXProperty *
+get_XProp (VObject *o)
+{
+	char *the_str;
+	CardXProperty *ret;
+	
+	ret = malloc(sizeof(CardXProperty)); 
+	ret->name = g_strdup (vObjectName (o));
+	ret->data = g_strdup (str_val (o));
+	free (the_str);
+	
+	return ret;
+}
+
 static CardRev 
 strtoCardRev(char *str)
 {
@@ -835,7 +900,7 @@ get_CardPhoto(VObject *o)
 		photo.size = vObjectIntegerValue(vo);
 		photo.data = malloc(photo.size);
 		memcpy (photo.data, vObjectAnyValue(o), photo.size);
-	} else if (vObjectValueType (o)) {
+	} else {
 		photo.size = strlen(str_val(o)) + 1;
 		photo.data = g_strdup(the_str);
 		free(the_str);
@@ -870,7 +935,7 @@ get_CardSound(VObject *o)
 		sound.size = vObjectIntegerValue(vo);
 		sound.data = malloc(sound.size);
 		memcpy (sound.data, vObjectAnyValue(o), sound.size);
-	} else if (vObjectValueType (o)) {
+	} else {
 		sound.size = strlen(str_val(o));
 		sound.data = g_strdup(the_str);
 		free(the_str);
@@ -900,11 +965,9 @@ card_create_from_vobject (VObject *vcrd)
 		
 		switch (propid) {
 		case PROP_FNAME:
-			if (vObjectValueType (o)) {
-				prop = &crd->fname.prop;
-				crd->fname.str = g_strdup(str_val(o));
-				free(the_str);
-			}
+			prop = &crd->fname.prop;
+			crd->fname.str = g_strdup(str_val(o));
+			free(the_str);
 			break;
 		 case PROP_NAME:
 			prop = &crd->name.prop;
@@ -915,149 +978,134 @@ card_create_from_vobject (VObject *vcrd)
 			crd->photo = get_CardPhoto(o);
 			break;
 		 case PROP_BDAY:
-			if (vObjectValueType (o)) {
-				prop = &crd->bday.prop;
-				crd->bday = strtoCardBDay(str_val(o));
-				free(the_str);
-			}
+			prop = &crd->bday.prop;
+			crd->bday = strtoCardBDay(str_val(o));
+			free(the_str);
 			break;
-		case PROP_DELADDR:
-			{
-				CardDelAddr *c;
-				c = get_CardDelAddr(o);
-				prop = &c->prop;
-				crd->deladdr.l = g_list_append(crd->deladdr.l, c);
-			}
+		 case PROP_DELADDR:
+				{
+					CardDelAddr *c;
+					c = get_CardDelAddr(o);
+					prop = &c->prop;
+					crd->deladdr.l = g_list_append(crd->deladdr.l, c);
+				}
 			break;
-		case PROP_DELLABEL:
-			if (vObjectValueType (o)) {
-				CardDelLabel *c;
-				c = get_CardDelLabel(o);
-				prop = &c->prop;
-				crd->dellabel.l = g_list_append(crd->dellabel.l, c);
-			}
+		 case PROP_DELLABEL:
+				{
+					CardDelLabel *c;
+					c = get_CardDelLabel(o);
+					prop = &c->prop;
+					crd->dellabel.l = g_list_append(crd->dellabel.l, c);
+				}
 			break;
-		case PROP_PHONE:
-			if (vObjectValueType (o)) {
-				CardPhone *c;
-				c = get_CardPhone(o);
-				prop = &c->prop;
-				crd->phone.l = g_list_append(crd->phone.l, c);
-			}
+		 case PROP_PHONE:
+				{
+					CardPhone *c;
+					
+					c = get_CardPhone(o);
+					prop = &c->prop;
+					crd->phone.l = g_list_append(crd->phone.l, c);
+				}
 			break;
-		case PROP_EMAIL:
-			if (vObjectValueType (o)) {
-				CardEMail *c;
-				c = get_CardEMail(o);
-				prop = &c->prop;
-				crd->email.l = g_list_append(crd->email.l, c);
-			}
+		 case PROP_EMAIL:
+				{
+					CardEMail *c;
+					
+					c = get_CardEMail(o);
+					prop = &c->prop;
+					crd->email.l = g_list_append(crd->email.l, c);
+				}
 			break;
-		case PROP_MAILER:
-			if (vObjectValueType (o)) {
-				prop = &crd->mailer.prop;
-				crd->mailer.str = g_strdup(str_val(o));
-				free(the_str);
-			}
+		 case PROP_MAILER:
+			prop = &crd->mailer.prop;
+			crd->mailer.str = g_strdup(str_val(o));
+			free(the_str);
 			break;
-		case PROP_TIMEZN:
-			if (vObjectValueType (o)) {
-				prop = &crd->timezn.prop;
-				crd->timezn = strtoCardTimeZone(str_val(o));
-				free(the_str);
-			}
+		 case PROP_TIMEZN:
+			prop = &crd->timezn.prop;
+			crd->timezn = strtoCardTimeZone(str_val(o));
+			free(the_str);
 			break;
-		case PROP_GEOPOS:
-			if (vObjectValueType (o)) {
-				prop = &crd->geopos.prop;
-				crd->geopos = strtoCardGeoPos(str_val(o));
-			}
+		 case PROP_GEOPOS:
+			prop = &crd->geopos.prop;
+			crd->geopos = strtoCardGeoPos(str_val(o));
 			break;
-		case PROP_TITLE:
-			if (vObjectValueType (o)) {
-				prop = &crd->title.prop;
-				crd->title.str = g_strdup(str_val(o));
-				free(the_str);
-			}
+		 case PROP_TITLE:
+			prop = &crd->title.prop;
+			crd->title.str = g_strdup(str_val(o));
+			free(the_str);
 			break;
-		case PROP_ROLE:
-			if (vObjectValueType (o)) {
-				prop = &crd->role.prop;
-				crd->role.str = g_strdup(str_val(o));
-				free(the_str);
-			}
+		 case PROP_ROLE:
+			prop = &crd->role.prop;
+			crd->role.str = g_strdup(str_val(o));
+			free(the_str);
 			break;
-		case PROP_LOGO:
+		 case PROP_LOGO:
 			prop = &crd->logo.prop;
 			crd->logo = get_CardPhoto(o);
 			break;
-		case PROP_AGENT:
+		 case PROP_AGENT:
 			crd->agent = card_create_from_vobject(o);
 			break;
-		case PROP_ORG:
+		 case PROP_ORG:
 			prop = &crd->org.prop;
 			crd->org = get_CardOrg(o);
 			break;
-		case PROP_CATEGORIES:
-			if (vObjectValueType (o)) {
-				prop = &crd->categories.prop;
-				crd->categories.str = g_strdup(str_val(o));
-				crd->categories.prop.encod = ENC_QUOTED_PRINTABLE;
-				free(the_str);
-			}
+		 case PROP_CATEGORIES:
+			prop = &crd->categories.prop;
+			crd->categories.str = g_strdup(str_val(o));
+			crd->categories.prop.encod = ENC_QUOTED_PRINTABLE;
+			free(the_str);
 			break;
-		case PROP_COMMENT:
-			if (vObjectValueType (o)) {
-				prop = &crd->comment.prop;
-				crd->comment.str = g_strdup(str_val(o));
-				crd->comment.prop.encod = ENC_QUOTED_PRINTABLE;
-				free(the_str);
-			}
+		 case PROP_COMMENT:
+			prop = &crd->comment.prop;
+			crd->comment.str = g_strdup(str_val(o));
+			crd->comment.prop.encod = ENC_QUOTED_PRINTABLE;
+			free(the_str);
 			break;
-		case PROP_REV:
-			if (vObjectValueType (o)) {
-				prop = &crd->rev.prop;
-				crd->rev = strtoCardRev(str_val(o));
-				free(the_str);
-			}
+		 case PROP_REV:
+			prop = &crd->rev.prop;
+			crd->rev = strtoCardRev(str_val(o));
+			free(the_str);
 			break;
 		 case PROP_SOUND:
 			prop = &crd->sound.prop;
 			crd->sound = get_CardSound(o);
 			break;
 		 case PROP_URL:
-			if (vObjectValueType (o)) {
-				prop = &crd->url.prop;
-				crd->url.str = g_strdup(str_val(o));
-				free(the_str);
-			}
+			prop = &crd->url.prop;
+			crd->url.str = g_strdup(str_val(o));
+			free(the_str);
 			break;
-		case PROP_UID:
-			if (vObjectValueType (o)) {
-				prop = &crd->uid.prop;
-				crd->uid.str = g_strdup(str_val(o));
-				free(the_str);
-			}
+		 case PROP_UID:
+			prop = &crd->uid.prop;
+			crd->uid.str = g_strdup(str_val(o));
+			free(the_str);
 			break;
 		 case PROP_VERSION:
-			if (vObjectValueType (o)) {
-				char *str;
-				str = str_val(o);
-				if (strcmp(str, "2.1"))
-					g_warning ("? < Version doesn't match.");
-				free(the_str);
-			}
+				{
+					char *str;
+					str = str_val(o);
+					if (strcmp(str, "2.1"))
+					  g_warning ("? < Version doesn't match.");
+					free(the_str);
+				}
 			break;
-		case PROP_KEY:
-			if (vObjectValueType (o)) {
-				prop = &crd->key.prop;
-				crd->key.type = get_key_type(o);
-				crd->key.data = g_strdup(str_val(o));
-				free(the_str);
-			}
+		 case PROP_KEY:
+			prop = &crd->key.prop;
+			crd->key.type = get_key_type(o);
+			crd->key.data = g_strdup(str_val(o));
+			free(the_str);
 			break;
 		 default:
-			g_warning("? < Property lookup returned invalid value.");
+				{
+					CardXProperty *c;
+				
+					c = get_XProp (o);
+					prop = &c->prop;
+					crd->xtension.l = g_list_append(crd->xtension.l, c);
+				}
+			break;
 		}
 		
 		if (prop) {
@@ -1112,6 +1160,8 @@ add_strProp(VObject *o, const char *id, char *val)
 static VObject *
 add_CardProperty(VObject *o, CardProperty *prop)
 {
+	GList *node;
+	
 	switch (prop->encod) {
 	 case ENC_BASE64:
 		addProp(o, VCBase64Prop);
@@ -1142,6 +1192,14 @@ add_CardProperty(VObject *o, CardProperty *prop)
 	 default:
 		g_warning("? < Card had invalid value type.");
 	}
+	
+	for (node = prop->xtension; node; node = node->next) {
+		CardXAttribute *xa = (CardXAttribute *) node->data;
+		if (xa->data)
+		  addPropValue (o, xa->name, xa->data);
+		else
+		  addProp (o, xa->name);
+	}
 
 	add_strProp(o, VCCharSetProp, prop->charset);
 	add_strProp(o, VCLanguageProp, prop->lang);
@@ -1191,6 +1249,29 @@ add_AddrType(VObject *o, int addr_type)
 	return o;
 }
 
+static void
+add_strAddrType(GString *string, int addr_type)
+{
+	int i, first = 1;
+	char *str;
+	
+	if (addr_type) {
+		g_string_append (string, " (");
+		
+		for (i = 0; addr_pairs[i].str; i++)
+		  if (addr_type & addr_pairs[i].id) {
+			  if (!first)
+			    g_string_append (string, ", ");
+			  first = 0;
+			  str = my_cap (addr_pairs[i].str);
+			  g_string_append (string, str);
+			  g_free (str);
+		  }
+		
+		g_string_append_c (string, ')');
+	}
+}
+
 static VObject *
 add_PhoneType(VObject *o, int phone_type)
 {
@@ -1203,6 +1284,29 @@ add_PhoneType(VObject *o, int phone_type)
 	return o;
 }
 
+static void
+add_strPhoneType(GString *string, int phone_type)
+{
+	int i, first = 1;
+	char *str;
+	
+	if (phone_type) {
+		g_string_append (string, " (");
+		
+		for (i = 0; phone_pairs[i].str; i++)
+		  if (phone_type & phone_pairs[i].id) {
+			  if (!first)
+			    g_string_append (string, ", ");
+			  first = 0;
+			  str = my_cap (phone_pairs[i].str);
+			  g_string_append (string, str);
+			  g_free (str);
+		  }
+		
+		g_string_append_c (string, ')');
+	}
+}
+
 static VObject *
 add_EMailType(VObject *o, enum EMailType email_type)
 {
@@ -1220,6 +1324,27 @@ add_EMailType(VObject *o, enum EMailType email_type)
 	return o;
 }
 
+static void
+add_strEMailType(GString *string, int email_type)
+{
+	int i;
+	char *str;
+	
+	if (email_type) {
+		g_string_append (string, " (");
+		
+		for (i = 0; email_pairs[i].str; i++)
+		  if (email_type == email_pairs[i].id) {
+			  str = my_cap (email_pairs[i].str);
+			  g_string_append (string, str);
+			  g_free (str);
+			  break;
+		  }
+		
+		g_string_append_c (string, ')');
+	}
+}
+
 static VObject *
 add_KeyType(VObject *o, enum KeyType key_type)
 {
@@ -1237,6 +1362,27 @@ add_KeyType(VObject *o, enum KeyType key_type)
 	return o;
 }
 
+static void
+add_strKeyType(GString *string, int key_type)
+{
+	int i;
+	char *str;
+	
+	if (key_type) {
+		g_string_append (string, " (");
+		
+		for (i = 0; key_pairs[i].str; i++)
+		  if (key_type == key_pairs[i].id) {
+			  str = my_cap (key_pairs[i].str);
+			  g_string_append (string, str);
+			  g_free (str);
+			  break;
+		  }
+		
+		g_string_append_c (string, ')');
+	}
+}
+
 static VObject *
 add_SoundType(VObject *o, enum SoundType sound_type)
 {
@@ -1314,6 +1460,17 @@ card_convert_to_vobject(Card *crd)
 		free(date_str);
 		add_CardProperty(vprop, &crd->bday.prop);
 	}
+
+	if (crd->xtension.l) {
+		GList *node;
+		
+		for (node = crd->xtension.l; node; node = node->next) {
+			CardXProperty *xp = (CardXProperty *) node->data;
+			addPropValue (vobj, xp->name, xp->data);
+			add_CardProperty(vobj, &xp->prop);
+		}
+	}
+			
 	
 	if (crd->deladdr.l) {
 		GList *node;
@@ -1450,8 +1607,225 @@ card_convert_to_vobject(Card *crd)
 	return vobj;
 }
 
+static void add_CardStrProperty_to_string (GString *string, char *prop_name,
+					   CardStrProperty *strprop)
+{
+	if (strprop->prop.used) {
+		if (prop_name)
+		  g_string_append (string, prop_name);
+		
+		g_string_append (string, strprop->str);
+	}
+}
+
+static void add_strProp_to_string (GString *string, char *prop_name, char *val)
+{
+	if (val) {
+		if (prop_name)
+		  g_string_append (string, prop_name);
+		
+		g_string_append (string, val);
+	}
+}
+
+static void addProp_to_string (GString *string, char *prop_name)
+{
+	if (prop_name)
+	  g_string_append (string, prop_name);
+}
+
 char *
 card_to_string (Card *crd)
+{
+	GString *string;
+	char *ret;
+	
+	string = g_string_new ("");
+	
+	add_CardStrProperty_to_string(string, _("Card: "), &crd->fname);
+	if (crd->name.prop.used) {
+		addProp_to_string(string, _("\nName: "));
+		add_strProp_to_string(string, _("\n  Prefix:     "), crd->name.prefix);
+		add_strProp_to_string(string, _("\n  Given:      "), crd->name.given);
+		add_strProp_to_string(string, _("\n  Additional: "), crd->name.additional);
+		add_strProp_to_string(string, _("\n  Family:     "), crd->name.family);
+		add_strProp_to_string(string, _("\n  Suffix:     "), crd->name.suffix);
+		g_string_append_c (string, '\n');
+	}
+	
+/*	if (crd->photo.prop.used) {
+		addPropSizedValue(string, _("\nPhoto: "), 
+					  crd->photo.data, crd->photo.size);
+		add_PhotoType(string, crd->photo.type);
+	}*/
+	
+	if (crd->bday.prop.used) {
+		char *date_str;
+		
+		date_str = card_bday_str(crd->bday);
+		add_strProp_to_string(string, _("\nBirth Date: "), date_str);
+		free(date_str);
+	}
+	
+	if (crd->deladdr.l) {
+		GList *node;
+		
+		for (node = crd->deladdr.l; node; node = node->next) {
+			CardDelAddr *deladdr = (CardDelAddr *) node->data;
+			
+			if (deladdr->prop.used) {
+				addProp_to_string(string, _("\nAddress:"));
+				add_strAddrType(string, deladdr->type);
+				add_strProp_to_string(string, _("\n  Postal Box:  "), deladdr->data[PO]);
+				add_strProp_to_string(string, _("\n  Ext:         "),deladdr->data[EXT]);
+				add_strProp_to_string(string, _("\n  Street:      "),deladdr->data[STREET]);
+				add_strProp_to_string(string, _("\n  City:        "), deladdr->data[CITY]);
+				add_strProp_to_string(string, _("\n  Region:      "), deladdr->data[REGION]);
+				add_strProp_to_string(string, _("\n  Postal Code: "), deladdr->data[CODE]);
+				add_strProp_to_string(string, _("\n  Country:     "), deladdr->data[COUNTRY]);
+			}
+		}
+		
+		g_string_append_c (string, '\n');
+	}
+	
+	if (crd->dellabel.l) {
+		GList *node;
+		
+		for (node = crd->dellabel.l; node; node = node->next) {
+			CardDelLabel *dellabel = (CardDelLabel *) node->data;
+			
+			add_strProp_to_string(string, _("\nDelivery Label: "),
+					    dellabel->data);
+			add_strAddrType(string, dellabel->type);
+		}
+	}
+	
+	if (crd->phone.l) {
+		GList *node;
+		char *sep;
+		
+		if (crd->phone.l->next) {
+			sep = "  ";
+			g_string_append (string, _("\nTelephones:\n"));
+		} else {
+			sep = " ";
+			g_string_append (string, _("\nTelephone:"));
+		}
+		
+		for (node = crd->phone.l; node; node = node->next) {
+			CardPhone *phone = (CardPhone *) node->data;
+
+			if (phone->prop.used) {
+				g_string_append (string, sep);
+				g_string_append (string, phone->data);
+				add_strPhoneType(string, phone->type);
+				g_string_append_c (string, '\n');
+			}
+		}
+		
+		if (crd->phone.l->next)
+		  g_string_append_c (string, '\n');
+	}
+
+	if (crd->email.l) {
+		GList *node;
+		char *sep;
+		
+		if (crd->email.l->next) {
+			sep = "  ";
+			g_string_append (string, _("\nE-mail:\n"));
+		} else {
+			sep = " ";
+			g_string_append (string, _("\nE-mail:"));
+		}
+		
+		
+		for (node = crd->email.l; node; node = node->next) {
+			CardEMail *email = (CardEMail *) node->data;
+			
+			if (email->prop.used) {
+				g_string_append (string, sep);
+				g_string_append (string, email->data);
+				add_strEMailType(string, email->type);
+				g_string_append_c (string, '\n');
+			}
+		}
+		
+		if (crd->email.l->next)
+		  g_string_append_c (string, '\n');
+	}
+
+	add_CardStrProperty_to_string(string, _("\nMailer: "), &crd->mailer);
+	
+	if (crd->timezn.prop.used) {
+		char *str;
+		
+		str = card_timezn_str(crd->timezn);
+		add_strProp_to_string(string, _("\nTime Zone: "), str);
+		free(str);
+	}
+	
+	if (crd->geopos.prop.used) {
+		char *str;
+		
+		str = card_geopos_str(crd->geopos);
+		add_strProp_to_string(string, _("\nGeo Location: "), str);
+		free(str);
+	}
+	
+        add_CardStrProperty_to_string(string, _("\nTitle: "), &crd->title);
+        add_CardStrProperty_to_string(string, _("\nBusiness Role: "), &crd->role);
+	
+/*	if (crd->logo.prop.used) {
+		addPropSizedValue(string, _("\nLogo: "), 
+					  crd->logo.data, crd->logo.size);
+		add_PhotoType(string, crd->logo.type);
+	}*/
+	
+/*	if (crd->agent)
+	  addstringectProp(string, card_convert_to_stringect(crd->agent));*/
+	
+	if (crd->org.prop.used) {
+		addProp_to_string(string, _("\nOrg: "));
+		add_strProp_to_string(string, _("\n  Name:  "), crd->org.name);
+		add_strProp_to_string(string, _("\n  Unit:  "), crd->org.unit1);
+		add_strProp_to_string(string, _("\n  Unit2: "), crd->org.unit2);
+		add_strProp_to_string(string, _("\n  Unit3: "), crd->org.unit3);
+		add_strProp_to_string(string, _("\n  Unit4: "), crd->org.unit4);
+		g_string_append_c (string, '\n');
+	}
+	
+        add_CardStrProperty_to_string(string, _("\nCategories: "), &crd->categories);
+        add_CardStrProperty_to_string(string, _("\nComment: "), &crd->comment);
+	
+/*	if (crd->sound.prop.used) {
+		if (crd->sound.type != SOUND_PHONETIC)
+		  addPropSizedValue(string, _("\nPronunciation: "),
+					    crd->sound.data, crd->sound.size);
+		else
+		  add_strProp_to_string(string, _("\nPronunciation: "), 
+				       crd->sound.data);
+		
+		add_SoundType(string, crd->sound.type);
+	}*/
+	
+        add_CardStrProperty_to_string(string, _("\nURL: "), &crd->url);
+        add_CardStrProperty_to_string(string, _("\nUnique String: "), &crd->uid);
+	
+	if (crd->key.prop.used) {
+		add_strProp_to_string(string, _("\nPublic Key: "), crd->key.data);
+		add_strKeyType(string, crd->key.type);
+	}
+	
+	ret = g_strdup (string->str);
+	g_string_free (string, TRUE);
+	
+	return ret;
+}
+
+char *
+card_to_vobj_string (Card *crd)
 {
 	VObject *object;
 	char *data, *ret_val;
