@@ -144,7 +144,7 @@ DFARS 252.227-7013 or 48 CFR 52.227-19, as applicable.
 
 /****  Types, Constants  ****/
 
-#define YYDEBUG		0	/* 1 to compile in some debugging code */
+#define YYDEBUG		1	/* 1 to compile in some debugging code */
 #define MAXTOKEN	256	/* maximum token (line) length */
 #define YYSTACKSIZE 	50	/* ~unref ? */
 #define MAXLEVEL	10	/* max # of nested objects parseable */
@@ -280,9 +280,9 @@ static const short yyrhs[] = {    22,
 static const short yyrline[] = { 0,
    207,   210,   213,   213,   217,   218,   221,   227,   232,   238,
    244,   245,   248,   252,   258,   261,   266,   266,   272,   273,
-   276,   279,   283,   290,   293,   294,   294,   298,   299,   303,
-   307,   309,   312,   315,   316,   319,   321,   322,   325,   332,
-   337,   343,   349,   356,   361,   367
+   276,   279,   283,   290,   293,   294,   294,   298,   299,   302,
+   306,   308,   311,   314,   315,   318,   320,   321,   324,   331,
+   336,   342,   348,   355,   360,   366
 };
 #endif
 
@@ -995,76 +995,76 @@ case 27:
 { enterValues(yyvsp[0].str); ;
     break;}
 case 29:
-#line 300 "vcc.y"
+#line 299 "vcc.y"
 { yyval.str = 0; ;
     break;}
 case 30:
-#line 305 "vcc.y"
+#line 304 "vcc.y"
 { if (!pushVObject(VCCalProp)) YYERROR; ;
     break;}
 case 31:
-#line 308 "vcc.y"
+#line 307 "vcc.y"
 { yyval.vobj = popVObject(); ;
     break;}
 case 32:
-#line 310 "vcc.y"
+#line 309 "vcc.y"
 { if (!pushVObject(VCCalProp)) YYERROR; ;
     break;}
 case 33:
-#line 312 "vcc.y"
+#line 311 "vcc.y"
 { yyval.vobj = popVObject(); ;
     break;}
 case 39:
-#line 327 "vcc.y"
+#line 326 "vcc.y"
 {
 	lexPushMode(L_VEVENT);
 	if (!pushVObject(VCEventProp)) YYERROR;
 	;
     break;}
 case 40:
-#line 333 "vcc.y"
+#line 332 "vcc.y"
 {
 	lexPopMode(0);
 	popVObject();
 	;
     break;}
 case 41:
-#line 338 "vcc.y"
+#line 337 "vcc.y"
 {
 	lexPushMode(L_VEVENT);
 	if (!pushVObject(VCEventProp)) YYERROR;
 	;
     break;}
 case 42:
-#line 343 "vcc.y"
+#line 342 "vcc.y"
 {
 	lexPopMode(0);
 	popVObject();
 	;
     break;}
 case 43:
-#line 351 "vcc.y"
+#line 350 "vcc.y"
 {
 	lexPushMode(L_VTODO);
 	if (!pushVObject(VCTodoProp)) YYERROR;
 	;
     break;}
 case 44:
-#line 357 "vcc.y"
+#line 356 "vcc.y"
 {
 	lexPopMode(0);
 	popVObject();
 	;
     break;}
 case 45:
-#line 362 "vcc.y"
+#line 361 "vcc.y"
 {
 	lexPushMode(L_VTODO);
 	if (!pushVObject(VCTodoProp)) YYERROR;
 	;
     break;}
 case 46:
-#line 367 "vcc.y"
+#line 366 "vcc.y"
 {
 	lexPopMode(0);
 	popVObject();
@@ -1292,7 +1292,7 @@ yyerrhandle:
     }
   return 1;
 }
-#line 373 "vcc.y"
+#line 372 "vcc.y"
 
 /*/////////////////////////////////////////////////////////////////////////*/
 static int pushVObject(const char *prop)
@@ -1714,7 +1714,7 @@ void initLex(const char *inputstring, unsigned long inputlen, CFile *inputfile)
 void initLex(const char *inputstring, unsigned long inputlen, FILE *inputfile)
 #endif
     {
-	    /* initialize lex mode stack */
+    /* initialize lex mode stack */
     lexBuf.lexModeStack[lexBuf.lexModeStackTop=0] = L_NORMAL;
 
     /* iniatialize lex buffer. */
@@ -1930,7 +1930,9 @@ static int yylex() {
 	if (c == ';') {
 	    DBG_(("db: SEMICOLON\n"));
 	    lexPushLookaheadc(c);
+#ifdef _SUPPORT_LINE_FOLDING
 	    handleMoreRFC822LineBreak(c);
+#endif		
 	    lexSkipLookahead();
 	    return SEMICOLON;
 	    }
@@ -1981,12 +1983,12 @@ static int yylex() {
 		case ':': {
 		    /* consume all line separator(s) adjacent to each other */
 		    /* ignoring linesep immediately after colon. */
-		    c = lexLookahead();
+/*		    c = lexLookahead();
 		    while (strchr("\n",c)) {
 			lexSkipLookahead();
 			c = lexLookahead();
 			++mime_lineNum;
-			}
+			}*/
 		    DBG_(("db: COLON\n"));
 		    return COLON;
 		    }
